@@ -5,22 +5,91 @@
 */
 
 #include "inputmethod_v1_interface.h"
-#include "seat_interface.h"
 #include "display.h"
-#include "surface_interface.h"
+#include "keyboard_interface.h"
+#include "keyboard_interface_p.h"
+#include "logging.h"
 #include "output_interface.h"
+#include "seat_interface.h"
+#include "surface_interface.h"
 #include "surfacerole_p.h"
 
 #include <QHash>
+#include <QTemporaryFile>
+
+#include <unistd.h>
 
 #include "qwayland-server-input-method-unstable-v1.h"
 #include "qwayland-server-text-input-unstable-v1.h"
+#include "qwayland-server-wayland.h"
 
 namespace KWaylandServer
 {
-
 static int s_version = 1;
 
+class InputKeyboardV1InterfacePrivate : public QtWaylandServer::wl_keyboard
+{
+public:
+    InputKeyboardV1InterfacePrivate()
+    {
+    }
+};
+
+InputMethodGrabV1::InputMethodGrabV1(QObject *parent)
+    : QObject(parent)
+    , d(new InputKeyboardV1InterfacePrivate)
+{
+}
+
+InputMethodGrabV1::~InputMethodGrabV1()
+{
+}
+
+void InputMethodGrabV1::sendKeymap(const QByteArray &keymap)
+{
+    QScopedPointer<QTemporaryFile> tmp(new QTemporaryFile());
+    if (!tmp->open()) {
+        qCWarning(KWAYLAND_SERVER) << "Failed to create keymap file:" << tmp->errorString();
+        return;
+    }
+
+    unlink(tmp->fileName().toUtf8().constData());
+    if (!tmp->resize(keymap.size())) {
+        qCWarning(KWAYLAND_SERVER) << "Failed to resize keymap file:" << tmp->errorString();
+        return;
+    }
+
+    uchar *address = tmp->map(0, keymap.size());
+    if (!address) {
+        qCWarning(KWAYLAND_SERVER) << "Failed to map keymap file:" << tmp->errorString();
+        return;
+    }
+
+    qstrncpy(reinterpret_cast<char *>(address), keymap.constData(), keymap.size() + 1);
+    tmp->unmap(address);
+
+    const auto resources = d->resourceMap();
+    for (auto r : resources) {
+        d->send_keymap(r->handle, QtWaylandServer::wl_keyboard::keymap_format::keymap_format_xkb_v1, tmp->handle(), tmp->size());
+    }
+}
+
+void InputMethodGrabV1::sendKey(quint32 serial, quint32 timestamp, quint32 key, KeyboardKeyState state)
+{
+    const auto resources = d->resourceMap();
+    for (auto r : resources) {
+        d->send_key(r->handle, serial, timestamp, key, quint32(state));
+    }
+}
+
+void InputMethodGrabV1::sendModifiers(quint32 serial, quint32 depressed, quint32 latched, quint32 locked, quint32 group)
+{
+    const auto resources = d->resourceMap();
+    for (auto r : resources) {
+        d->send_modifiers(r->handle, depressed, latched, locked, group, serial);
+    }
+}
+
 class InputMethodContextV1InterfacePrivate : public QtWaylandServer::zwp_input_method_context_v1
 {
 public:
@@ -30,6 +99,10 @@ public:
     {
     }
 
+    ~InputMethodContextV1InterfacePrivate()
+    {
+    }
+
     void zwp_input_method_context_v1_commit_string(Resource *, uint32_t serial, const QString &text) override
     {
         Q_EMIT q->commitString(serial, text);
@@ -80,15 +153,22 @@ public:
     {
         Q_EMIT q->keysym(serial, time, sym, state == WL_KEYBOARD_KEY_STATE_PRESSED, toQtModifiers(modifiers));
     }
-    void zwp_input_method_context_v1_grab_keyboard(Resource *, uint32_t keyboard) override
+    void zwp_input_method_context_v1_grab_keyboard(Resource *resource, uint32_t id) override
     {
-        Q_EMIT q->grabKeyboard(keyboard);
+        m_keyboardGrab.reset(new InputMethodGrabV1(q));
+        m_keyboardGrab->d->add(resource->client(), id, 1);
+        Q_EMIT q->keyboardGrabRequested(m_keyboardGrab.data());
     }
     void zwp_input_method_context_v1_key(Resource *, uint32_t serial, uint32_t time, uint32_t key, uint32_t state) override
     {
         Q_EMIT q->key(serial, time, key, state == WL_KEYBOARD_KEY_STATE_PRESSED);
     }
-    void zwp_input_method_context_v1_modifiers(Resource *, uint32_t serial, uint32_t mods_depressed, uint32_t mods_latched, uint32_t mods_locked, uint32_t group) override
+    void zwp_input_method_context_v1_modifiers(Resource *,
+                                               uint32_t serial,
+                                               uint32_t mods_depressed,
+                                               uint32_t mods_latched,
+                                               uint32_t mods_locked,
+                                               uint32_t group) override
     {
         Q_EMIT q->modifiers(serial, mods_depressed, mods_latched, mods_locked, group);
     }
@@ -109,6 +189,9 @@ public:
         case ZWP_TEXT_INPUT_V1_TEXT_DIRECTION_AUTO:
             qtDirection = Qt::LayoutDirectionAuto;
             break;
+        default:
+            Q_UNREACHABLE();
+            break;
         }
         Q_EMIT q->textDirection(serial, qtDirection);
     }
@@ -131,8 +214,8 @@ public:
         wl_resource_destroy(resource->handle);
     }
 
-private:
     InputMethodContextV1Interface *const q;
+    QScopedPointer<InputMethodGrabV1> m_keyboardGrab;
     QVector<Qt::KeyboardModifiers> mods;
 };
 
@@ -265,9 +348,15 @@ void InputMethodContextV1Interface::sendSurroundingText(const QString &text, uin
     }
 }
 
+InputMethodGrabV1 *InputMethodContextV1Interface::keyboardGrab() const
+{
+    return d->m_keyboardGrab.get();
+}
+
 class InputPanelSurfaceV1InterfacePrivate : public QtWaylandServer::zwp_input_panel_surface_v1, public SurfaceRole
 {
     friend class InputPanelSurfaceV1Interface;
+
 public:
     InputPanelSurfaceV1InterfacePrivate(SurfaceInterface *surface, quint32 id, InputPanelSurfaceV1Interface *q)
         : zwp_input_panel_surface_v1()
@@ -287,9 +376,12 @@ public:
         Q_EMIT q->topLevel(OutputInterface::get(output), InputPanelSurfaceV1Interface::Position(position));
     }
 
-    void commit() override {}
+    void commit() override
+    {
+    }
 
-    void zwp_input_panel_surface_v1_destroy_resource(Resource *) override {
+    void zwp_input_panel_surface_v1_destroy_resource(Resource *) override
+    {
         delete q;
     }
 
@@ -321,9 +413,7 @@ public:
 
         SurfaceRole *surfaceRole = SurfaceRole::get(surface);
         if (surfaceRole) {
-            wl_resource_post_error(resource->handle, 0,
-                                   "the surface already has a role assigned %s",
-                                   surfaceRole->name().constData());
+            wl_resource_post_error(resource->handle, 0, "the surface already has a role assigned %s", surfaceRole->name().constData());
             return;
         }
 
@@ -369,7 +459,7 @@ public:
         send_activate(resource->handle, addedResource->handle);
     }
 
-    InputMethodContextV1Interface *m_context = nullptr;
+    QScopedPointer<InputMethodContextV1Interface> m_context;
     InputMethodV1Interface *const q;
     Display *const m_display;
 };
@@ -388,7 +478,7 @@ void InputMethodV1Interface::sendActivate()
         return;
     }
 
-    d->m_context = new InputMethodContextV1Interface(this);
+    d->m_context.reset(new InputMethodContextV1Interface(this));
 
     for (auto resource : d->resourceMap()) {
         auto connection = d->m_context->d->add(resource->client(), resource->version());
@@ -408,13 +498,12 @@ void InputMethodV1Interface::sendDeactivate()
             d->send_deactivate(resource->handle, connection->handle);
         }
     }
-    delete d->m_context;
-    d->m_context = nullptr;
+    d->m_context.reset();
 }
 
 InputMethodContextV1Interface *InputMethodV1Interface::context() const
 {
-    return d->m_context;
+    return d->m_context.get();
 }
 
 }
