@@ -15,7 +15,6 @@ struct wl_resource;
 
 namespace KWaylandServer
 {
-
 class Display;
 class OutputInterface;
 class SeatInterface;
@@ -113,8 +112,7 @@ public:
     /**
      * Constructs an XdgSurfaceInterface for the given \a shell and \a surface.
      */
-    XdgSurfaceInterface(XdgShellInterface *shell, SurfaceInterface *surface,
-                        ::wl_resource *resource);
+    XdgSurfaceInterface(XdgShellInterface *shell, SurfaceInterface *surface, ::wl_resource *resource);
     /**
      * Destructs the XdgSurfaceInterface object.
      */
@@ -162,6 +160,11 @@ public:
     static XdgSurfaceInterface *get(::wl_resource *resource);
 
 Q_SIGNALS:
+    /**
+     * This signal is emitted when the xdg-surface is about to be destroyed.
+     */
+    void aboutToBeDestroyed();
+
     /**
      * This signal is emitted when a configure event with serial \a serial has been acknowledged.
      */
@@ -280,6 +283,11 @@ public:
     static XdgToplevelInterface *get(::wl_resource *resource);
 
 Q_SIGNALS:
+    /**
+     * This signal is emitted when the xdg-toplevel is about to be destroyed.
+     */
+    void aboutToBeDestroyed();
+
     /**
      * This signal is emitted when the xdg-toplevel has commited the initial state and wants to
      * be configured. After initializing the toplevel, you must send a configure event.
@@ -481,8 +489,7 @@ class KWAYLANDSERVER_EXPORT XdgPopupInterface : public QObject
     Q_OBJECT
 
 public:
-    XdgPopupInterface(XdgSurfaceInterface *surface, SurfaceInterface *parentSurface,
-                      const XdgPositioner &positioner, ::wl_resource *resource);
+    XdgPopupInterface(XdgSurfaceInterface *surface, SurfaceInterface *parentSurface, const XdgPositioner &positioner, ::wl_resource *resource);
     /**
      * Destructs the XdgPopupInterface object.
      */
@@ -537,6 +544,11 @@ public:
     static XdgPopupInterface *get(::wl_resource *resource);
 
 Q_SIGNALS:
+    /**
+     * This signal is emitted when the xdg-popup is about to be destroyed.
+     */
+    void aboutToBeDestroyed();
+
     /**
      * This signal is emitted when the xdg-popup has commited the initial state and wants to
      * be configured. After initializing the popup, you must send a configure event.
